#coding: utf-8
require_relative '../lib/banana_docs'

describe BananaDocs::Helpers do 
  subject(:banana_docs_helpers) { described_class.new }
  subject(:navigations) { banana_docs_helpers.instance_variable_get(:@navigations) }
  subject(:fixed_columns) { described_class::FIXED_COLUMNS }

  it { expect(banana_docs_helpers).to be_instance_of(described_class) }

  it "all fixed columns" do 
    expect(fixed_columns).to include('Parâmetro')
    expect(fixed_columns).to include('Descrição')
    expect(fixed_columns).to include('Tipo')
    expect(fixed_columns).to include('Formato')
    expect(fixed_columns).to include('Observações')
    expect(fixed_columns).to include('Requerido?')
  end

  context "when create a new instance" do 
    it "helpers.navigations need be empty" do 
      expect(navigations).to be_empty
    end
  end

  context "helpers methods" do 
    let(:navigation) { { title: 'Title', url: 'example.org' } }
    let(:section) { banana_docs_helpers.section('New Section') }
    let(:action) { banana_docs_helpers.action('New Action', method: 'POST', url: '/url/action') }
    let(:param) { banana_docs_helpers.param(:param, 'It is a new param', :string, 'string_format') }
    let(:required_true) { banana_docs_helpers.param(:param, 'It is a new param', :string, 'string_format', required: true) }
    let(:required_false) { banana_docs_helpers.param(:param, 'It is a new param', :integer, 'integer_format', required: false) }
    let(:link) { banana_docs_helpers.link('New Link', name: 'new-link') } 
    let(:menu_nav) { banana_docs_helpers.show_nav_menu }
    let(:span) { banana_docs_helpers.build_span('Sim', 'badge badge-important') }
    let(:note) { banana_docs_helpers.note('Lero Lero') }
    let(:key_value) { banana_docs_helpers.key_value('key', 'value') }
    let(:status_code) { banana_docs_helpers.status_code(200) }

    it "#add_nav_item" do 
      expect{
        navigations << navigation
      }.to change { navigations.size }.by(1)
      # OR
      # expect{ 
      #   navigations << navigation
      # }.to change (navigations, :size).by(1)
    end

    it "#section - add new section" do 
      expect(section).to include('New Section')
      expect(section).to include('new-section')
    end

    it "#action - add new action" do 
      expect(action).to include('New Action')
      expect(action).to include('POST')
      expect(action).to include('/url/action')
    end

    it "#build_span" do 
      expect(span).to include('Sim')
      expect(span).to include('badge badge-important')
    end

    it "#note" do 
      expect(note).to include('Lero Lero')
    end

    it "#status_code" do 
      expect(status_code).to include('200')
    end

    context "#param" do 

      context "without options" do 
        it "default" do 
          expect(param).to include('param')
          expect(param).to include('It is a new param')
          expect(param).to include('string')
        end
      end
 
      context "with options" do
        it "required is false" do 
          expect(required_false).to include('param')
          expect(required_false).to include('It is a new param')
          expect(required_false).to include('integer')
        end

        it "required is true" do 
          expect(required_true).to include('param')
          expect(required_true).to include('It is a new param')
          expect(required_true).to include('string')
        end
      end
    end

    context "#show_nav_menu" do 
      it "default navigation" do 
        expect(menu_nav).to include("<ul class=\"nav nav-list\"><li class=\"nav-header\">A&ccedil;&otilde;es</li></ul>")
      end
      it "add a item" do 
        banana_docs_helpers.add_nav_item('New Link', 'new-link')
        expect(menu_nav).to include("<ul class=\"nav nav-list\"><li class=\"nav-header\">A&ccedil;&otilde;es</li><li><a href=\"#new-link\">New Link</a></li></ul>")
      end
    end

    context "#key_value" do 
      it "include key/value" do 
        expect(key_value).to include('key')
        expect(key_value).to include('value')
        expect(key_value).to include("{ \"key\":\"value\" }")
      end
    end

  end

end